#ifndef DOMAIN_ATTRIBUTE_H
#define DOMAIN_ATTRIBUTE_H

#include <utility>
#include <vector>

#include "rule.h"
#include "value.h"

namespace domain {

    class Enumeration {
        std::string name;
        Value value;
    public:
        Enumeration(std::string name, Value value);

        std::string &getName();

        Value getValue();
    };

    class Popup {
    public:
        enum PopupType {
            AUTOMATIC,
            ON_DEMAND
        };

    private:
        std::string title;
        std::string text;
        PopupType type;

        bool enumerated;
        std::vector<Enumeration> enumeration;
        std::vector<Rule> rules;

    public:
        Popup(std::string title, std::string text, PopupType type, std::vector<Rule> rules,
              std::vector<Enumeration> enumeration);

        std::string getTitle();

        std::string getText();

        PopupType getType();

        std::vector<Rule> getRules();

        bool isEnumerated();

        std::vector<Enumeration> &getEnumeration();

        void setEnumeration(std::vector<Enumeration> enums) {
            enumerated = true;
            enumeration = enums;
        }

    };

    class Attribute {
        std::string name;
        std::optional<std::string> displayName;
        Value defaultValue;
        std::optional<Popup> popup;
    public:
        Attribute(std::string name, std::optional<std::string> displayName, Value defaultValue, std::optional<Popup> popup = std::nullopt);

        std::string getName();

        std::string getDisplayName();

        Value getDefault();

        std::optional<Popup> getPopup();
        void setPupup(std::optional<Popup> popup);
    };

} // namespace domain

#endif // DOMAIN_ATTRIBUTE_H
